class BlogController < ApplicationController
  include BlogHelper

# TODO 全文検索機能．．．by keisen
# FIXME データがゼロ件のときにエラーが出る．．．by keisen

# -- filters ----------------------------------------------------------------- #
  before_filter :load_configuration,
                :load_sidebar,
                :only => [:index, :day, :month, :rss, :search]

# -- cache sweepers ---------------------------------------------------------- #
  caches_page :index, :day, :month, :rss
  cache_sweeper :comment_sweeper, :only => [:create_comment]
  cache_sweeper :trackback_sweeper, :only => [:tb]

# -- main view --------------------------------------------------------------- #
  def index
    @pages, @diaries = paginate :diaries,
                                :select => 'id, date',
                                :per_page => @config.per_page,
                                :order => 'date desc'
  end

  def day
    unless params[:day]
      params[:day] = "#{params[:y]}-#{params[:m]}-#{params[:d]}"
    end
    @diaries = Array.new
    @diaries << Diary.find(:first,
                           :select => 'id, date',
                           :order => 'date desc',
                           :conditions => ["date = ?", params[:day][0, 10]])
    if @diaries.nil? || @diaries[0].nil?
      redirect_to :controller => :blog, :action => :index
      return true
    end
    @prev_diary = Diary.find :first,
                             :select => :date,
                             :order => 'date',
                             :conditions => ["date > ?", @diaries[0].date]
    @next_diary = Diary.find :first,
                             :select => :date,
                             :order => 'date desc',
                             :conditions => ["date < ?", @diaries[0].date]
  end

  def month
    first = Date.strptime params[:month].to_s + '-1', '%Y-%m-%d'
    last = Date.new first.year, first.month, -1
    @diaries = Diary.find :all,
                          :select => 'id, date',
                          :order => 'date desc',
                          :conditions => ["date >= ? and date <= ?", first, last]
    @prev_diary = Diary.find :first,
                             :select => :date,
                             :order => 'date',
                             :conditions => ["date > ?", last]
    @next_diary = Diary.find :first,
                             :select => :date,
                             :order => 'date desc',
                             :conditions => ["date < ?", first]
  end

# -- comment ----------------------------------------------------------------- #
  def new_comment
    @diary_id = params[:id]
    render :partial => "comment_input"
  end

  def preview_comment
    @comment = Comment.new params[:comment]
    render :partial => "comment"
  end

  def create_comment
    comment = Comment.new params[:comment]
    comment.save
    redirect_to :back
  end

# -- rss --------------------------------------------------------------------- #
  def rss
    # TODO 日付単位で出力し，Bodyにエントリーと概略を出力する機能を追加．．．by keisen
    rss = RSS::Maker.make "1.0" do |maker|
      entries = Entry.find :all,
                           :select => "id, diary_id, title",
                           :limit => 15,
                           :order => 'created_at desc'
      for entry in entries do
        date = entry.diary.date.to_s
        item  = maker.items.new_item
        item.link = url_for :controller => 'blog',
                            :action => 'day',
                            :day => date,
                            :anchor => 'e' + entry.id.to_s
        item.title = entry.title
        item.date = date
      end
      maker.channel.link        = url_for :action=>'index', :page=>'1'
      maker.channel.about       = url_for :action=>'index', :page=>'1'
      maker.channel.title       = @config.title
      maker.channel.description = @config.title
    end
    @headers["Content-Type"] = 'application/xml; charset=UTF-8'
    render :text => rss.to_s, :layout => false
  end

# -- trackback --------------------------------------------------------------- #
  def tb
    # FIXME DiaryがDBに存在しない場合，登録処理を実行しないようにして欲しいです．．．by keisen
#    if params[:__mode] == "rss"
#      # TODO rss生成
#      return
#    end
    diary = Diary.find_by_date params[:day]
    trackback = Trackback.new({:title => params[:title].toutf8,
                                :excerpt => params[:excerpt].toutf8,
                                :url => params[:url],
                                :blog_name => params[:blog_name].toutf8,
                                :diary_id => diary.id})
    t = Thread.new trackback do |trackback|
      begin
        trackback.save
      rescue => ex
        logger.error "Error:" + ex
      end
    end
    render :text => 'trackback', :layout => false
  end

# -- search ------------------------------------------------------------------ #
  def search
    @pages, @entries = paginate :entries,
                                :per_page => 5,
                                :select => "entries.id, entries.diary_id",
                                :order => "entries.created_at desc" if params[:keywords].nil?
    @keywords = ""
    params[:keywords].split(/　|\s/).each do |keyword|
      ngram_word = keyword.ngram(2, "+")
      @keywords << ngram_word if ngram_word
      @keywords << " "
    end
    @keywords.strip
    @pages, @entries = paginate :entries,
                                :per_page => 5, 
                                :select => "entries.id, entries.diary_id",
                                :join => 'inner join ngram_texts t on entries.id = t.entry_id',
                                :conditions => ["match(t.title,t.body) against(? in boolean mode)", @keywords],
                                :order => "entries.created_at desc"                                
  end

# -- protected --------------------------------------------------------------- #
  protected
  def load_configuration
    @config = EmeryConfig.find :first
  end

  protected
  def load_sidebar
    return true if read_fragment :side_contents

    unless read_fragment :new_entries
      new_entries_limit = @config.new_entries_limit
      @new_entries = Entry.find :all,
                                :select => "id, diary_id, title",
                                :limit => new_entries_limit,
                                :order => 'created_at desc'
    end

    unless read_fragment :new_comments
      # FIXME ５日分のコメントを出力するようにしたい．．．by keisen
      new_comments_limit = @config.new_comments_limit
      @new_comments = Comment.find :all,
                                   :select => "id, diary_id, author",
                                   :limit => new_comments_limit,
                                   :order => 'created_at desc'
    end

    unless read_fragment :new_trackbacks
      @new_trackbacks = Trackback.find :all,
                                       :select => "id, title, url, blog_name, diary_id",
                                       :limit => 5,
                                       :order => 'created_at desc'
    end

    unless read_fragment :archives
      @archives = Array.new

      months = Diary.find(:all,
                      :select => 'MIN(DATE) date',
                      :order => "DATE_FORMAT(date,'%Y%m') DESC",
                      :group => "DATE_FORMAT(date,'%Y%m')" )

      months.each{|month|@archives << month.date} if months
    end
  end
end

